@@ -2,21 +2,17 @@
 """Tests the xonsh environment."""
 from __future__ import unicode_literals, print_function
 import os
-import tempfile
-import builtins
 import itertools
 from tempfile import TemporaryDirectory
-from xonsh.tools import ON_WINDOWS, always_true
+from xonsh.tools import always_true
 
 import pytest
 
 from xonsh.commands_cache import CommandsCache
 from xonsh.environ import (
     Env,
-    Ensurer,
     locate_binary,
-    DEFAULT_ENSURERS,
-    DEFAULT_VALUES,
+    DEFAULT_VARS,
     default_env,
     make_args_env,
     LsColors,
@@ -69,7 +65,7 @@ def test_env_detype_mutable_access_clear(path1, path2):
 
 def test_env_detype_no_dict():
     env = Env(YO={"hey": 42})
-    env.set_ensurer("YO", Ensurer(always_true, None, None))
+    env.register("YO", validate=always_true, convert=None, detype=None)
     det = env.detype()
     assert "YO" not in det
 
@@ -101,6 +97,14 @@ def test_histcontrol_ignoreerr_ignoredups():
     assert "ignoredups" in env["HISTCONTROL"]
 
 
+def test_histcontrol_ignoreerr_ignoredups_erase_dups():
+    env = Env(HISTCONTROL="ignoreerr,ignoredups,ignoreerr,erasedups")
+    assert len(env["HISTCONTROL"]) == 3
+    assert "ignoreerr" in env["HISTCONTROL"]
+    assert "ignoredups" in env["HISTCONTROL"]
+    assert "erasedups" in env["HISTCONTROL"]
+
+
 def test_swap():
     env = Env(VAR="wakka")
     assert env["VAR"] == "wakka"
@@ -165,6 +169,7 @@ def test_event_on_envvar_change(xonsh_builtins):
     xonsh_builtins.__xonsh__.env = env
     share = []
     # register
+
     @xonsh_builtins.events.on_envvar_change
     def handler(name, oldvalue, newvalue, **kwargs):
         share.extend((name, oldvalue, newvalue))
@@ -180,6 +185,7 @@ def test_event_on_envvar_new(xonsh_builtins):
     xonsh_builtins.__xonsh__.env = env
     share = []
     # register
+
     @xonsh_builtins.events.on_envvar_new
     def handler(name, value, **kwargs):
         share.extend((name, value))
@@ -195,6 +201,7 @@ def test_event_on_envvar_change_from_none_value(xonsh_builtins):
     xonsh_builtins.__xonsh__.env = env
     share = []
     # register
+
     @xonsh_builtins.events.on_envvar_change
     def handler(name, oldvalue, newvalue, **kwargs):
         share.extend((name, oldvalue, newvalue))
@@ -211,6 +218,7 @@ def test_event_on_envvar_change_no_fire_when_value_is_same(val, xonsh_builtins):
     xonsh_builtins.__xonsh__.env = env
     share = []
     # register
+
     @xonsh_builtins.events.on_envvar_change
     def handler(name, oldvalue, newvalue, **kwargs):
         share.extend((name, oldvalue, newvalue))
@@ -226,12 +234,13 @@ def test_events_on_envvar_called_in_right_order(xonsh_builtins):
     xonsh_builtins.__xonsh__.env = env
     share = []
     # register
+
     @xonsh_builtins.events.on_envvar_new
     def handler(name, value, **kwargs):
         share[:] = ["new"]
 
     @xonsh_builtins.events.on_envvar_change
-    def handler(name, oldvalue, newvalue, **kwargs):
+    def handler1(name, oldvalue, newvalue, **kwargs):
         share[:] = ["change"]
 
     # trigger new
@@ -245,13 +254,6 @@ def test_events_on_envvar_called_in_right_order(xonsh_builtins):
     assert share == ["change"]
 
 
-def test_int_bool_envvars_have_ensurers():
-    bool_ints = [type(envvar) in [bool, int] for envvar in DEFAULT_VALUES.values()]
-    key_mask = set(itertools.compress(DEFAULT_VALUES.keys(), bool_ints))
-    ensurer_keys = set(DEFAULT_ENSURERS.keys())
-    assert len(key_mask.intersection(ensurer_keys)) == len(key_mask)
-
-
 def test_no_lines_columns():
     os.environ["LINES"] = "spam"
     os.environ["COLUMNS"] = "eggs"
@@ -281,13 +283,13 @@ def test_delitem():
     assert env["VAR"] == "a value"
     del env["VAR"]
     with pytest.raises(Exception):
-        a = env["VAR"]
+        env["VAR"]
 
 
 def test_delitem_default():
     env = Env()
     a_key, a_value = next(
-        (k, v) for (k, v) in env._defaults.items() if isinstance(v, str)
+        (k, v.default) for (k, v) in env._vars.items() if isinstance(v.default, str)
     )
     del env[a_key]
     assert env[a_key] == a_value
@@ -295,7 +297,161 @@ def test_delitem_default():
     assert env[a_key] == a_value
 
 
-def test_lscolors_target():
+def test_lscolors_target(xonsh_builtins):
     lsc = LsColors.fromstring("ln=target")
-    assert lsc["ln"] == ("TARGET",)
+    assert lsc["ln"] == ("NO_COLOR",)
+    assert lsc.is_target("ln")
     assert lsc.detype() == "ln=target"
+    assert not (lsc.is_target("mi"))
+
+
+@pytest.mark.parametrize(
+    "key_in,old_in,new_in,test",
+    [
+        ("fi", ("NO_COLOR",), ("BLUE",), "existing key, change value"),
+        ("fi", ("NO_COLOR",), ("NO_COLOR",), "existing key, no change in value"),
+        ("tw", None, ("NO_COLOR",), "create new key"),
+        ("pi", ("BACKGROUND_BLACK", "YELLOW"), None, "delete existing key"),
+    ],
+)
+def test_lscolors_events(key_in, old_in, new_in, test, xonsh_builtins):
+    lsc = LsColors.fromstring("fi=0:di=01;34:pi=40;33")
+    # corresponding colors: [('NO_COLOR',), ('BOLD_CYAN',), ('BOLD_CYAN',), ('BACKGROUND_BLACK', 'YELLOW')]
+
+    event_fired = False
+
+    @xonsh_builtins.events.on_lscolors_change
+    def handler(key, oldvalue, newvalue, **kwargs):
+        nonlocal old_in, new_in, key_in, event_fired
+        assert (
+            key == key_in and oldvalue == old_in and newvalue == new_in
+        ), "Old and new event values match"
+        event_fired = True
+
+    xonsh_builtins.__xonsh__.env["LS_COLORS"] = lsc
+
+    if new_in is None:
+        lsc.pop(key_in, "argle")
+    else:
+        lsc[key_in] = new_in
+
+    if old_in == new_in:
+        assert not event_fired, "No event if value doesn't change"
+    else:
+        assert event_fired
+
+
+def test_register_custom_var_generic():
+    """Test that a registered envvar without any type is treated
+    permissively.
+
+    """
+    env = Env()
+
+    assert "MY_SPECIAL_VAR" not in env
+    env.register("MY_SPECIAL_VAR")
+    assert "MY_SPECIAL_VAR" in env
+
+    env["MY_SPECIAL_VAR"] = 32
+    assert env["MY_SPECIAL_VAR"] == 32
+
+    env["MY_SPECIAL_VAR"] = True
+    assert env["MY_SPECIAL_VAR"] == True
+
+
+def test_register_custom_var_int():
+    env = Env()
+    env.register("MY_SPECIAL_VAR", type='int')
+
+    env["MY_SPECIAL_VAR"] = "32"
+    assert env["MY_SPECIAL_VAR"] == 32
+
+    with pytest.raises(ValueError):
+        env["MY_SPECIAL_VAR"] = "wakka"
+
+
+def test_register_custom_var_float():
+    env = Env()
+    env.register("MY_SPECIAL_VAR", type='float')
+
+    env["MY_SPECIAL_VAR"] = "27"
+    assert env["MY_SPECIAL_VAR"] == 27.0
+
+    with pytest.raises(ValueError):
+        env["MY_SPECIAL_VAR"] = "wakka"
+
+
+@pytest.mark.parametrize("val,converted", 
+        [
+            (True, True),
+            (32, True),
+            (0, False),
+            (27.0, True),
+            (None, False),
+            ("lol", True),
+            ("false", False),
+            ("no", False),
+            ])
+def test_register_custom_var_bool(val, converted):
+    env = Env()
+    env.register("MY_SPECIAL_VAR", type='bool')
+
+    env["MY_SPECIAL_VAR"] = val
+    assert env["MY_SPECIAL_VAR"] == converted
+
+
+@pytest.mark.parametrize("val,converted", 
+        [
+            (32, "32"),
+            (0, "0"),
+            (27.0, "27.0"),
+            (None, "None"),
+            ("lol", "lol"),
+            ("false", "false"),
+            ("no", "no"),
+        ])
+def test_register_custom_var_str(val, converted):
+    env = Env()
+    env.register("MY_SPECIAL_VAR", type='str')
+
+    env["MY_SPECIAL_VAR"] = val
+    assert env["MY_SPECIAL_VAR"] == converted
+
+
+def test_register_custom_var_path():
+    env = Env()
+    env.register("MY_SPECIAL_VAR", type='path')
+
+    paths = ["/home/wakka", "/home/wakka/bin"]
+    env["MY_SPECIAL_VAR"] = paths
+
+    assert hasattr(env['MY_SPECIAL_VAR'], 'paths')
+    assert env["MY_SPECIAL_VAR"].paths == paths
+
+    with pytest.raises(TypeError):
+        env["MY_SPECIAL_VAR"] = 32
+    
+
+def test_deregister_custom_var():
+    env = Env()
+
+    env.register("MY_SPECIAL_VAR", type='path')
+    env.deregister("MY_SPECIAL_VAR")
+    assert "MY_SPECIAL_VAR" not in env
+
+    env.register("MY_SPECIAL_VAR", type='path')
+    paths = ["/home/wakka", "/home/wakka/bin"]
+    env["MY_SPECIAL_VAR"] = paths
+    env.deregister("MY_SPECIAL_VAR")
+
+    # deregistering a variable that has a value set doesn't
+    # remove it from env;
+    # the existing variable also maintains its type validation, conversion
+    assert "MY_SPECIAL_VAR" in env
+    with pytest.raises(TypeError):
+        env["MY_SPECIAL_VAR"] = 32
+
+    # removing, then re-adding the variable without registering
+    # gives it only default permissive validation, conversion
+    del env["MY_SPECIAL_VAR"]
+    env["MY_SPECIAL_VAR"] = 32
