@@ -1,6 +1,5 @@
 import os
 import sys
-import time
 import shutil
 import tempfile
 import subprocess as sp
@@ -8,13 +7,12 @@ import subprocess as sp
 import pytest
 
 import xonsh
-from xonsh.platform import ON_WINDOWS
 from xonsh.lib.os import indir
 
 from tools import (
+    skip_if_on_msys,
     skip_if_on_windows,
     skip_if_on_darwin,
-    skip_if_on_travis,
     ON_WINDOWS,
     ON_DARWIN,
     ON_TRAVIS,
@@ -56,32 +54,40 @@ skip_if_no_sleep = pytest.mark.skipif(
 )
 
 
-def run_xonsh(cmd, stdin=sp.PIPE, stdout=sp.PIPE, stderr=sp.STDOUT):
+def run_xonsh(cmd, stdin=sp.PIPE, stdout=sp.PIPE, stderr=sp.STDOUT, single_command=False):
     env = dict(os.environ)
     env["PATH"] = PATH
-    env["XONSH_DEBUG"] = "1"
+    env["XONSH_DEBUG"] = "0" # was "1"
     env["XONSH_SHOW_TRACEBACK"] = "1"
     env["RAISE_SUBPROC_ERROR"] = "0"
     env["PROMPT"] = ""
     xonsh = "xonsh.bat" if ON_WINDOWS else "xon.sh"
     xonsh = shutil.which(xonsh, path=PATH)
+    if single_command:
+        args = [xonsh, "--no-rc", "-c", cmd]
+        input = None
+    else:
+        args = [xonsh, "--no-rc"]
+        input = cmd
+
     proc = sp.Popen(
-        [xonsh, "--no-rc"],
+        args,
         env=env,
         stdin=stdin,
         stdout=stdout,
         stderr=stderr,
         universal_newlines=True,
     )
+
     try:
-        out, err = proc.communicate(input=cmd, timeout=10)
+        out, err = proc.communicate(input=input, timeout=10)
     except sp.TimeoutExpired:
         proc.kill()
         raise
     return out, err, proc.returncode
 
 
-def check_run_xonsh(cmd, fmt, exp):
+def check_run_xonsh(cmd, fmt, exp, exp_rtn=0):
     """The ``fmt`` parameter is a function
     that formats the output of cmd, can be None.
     """
@@ -91,7 +97,7 @@ def check_run_xonsh(cmd, fmt, exp):
     if callable(exp):
         exp = exp()
     assert out == exp, err
-    assert rtn == 0, err
+    assert rtn == exp_rtn, err
 
 
 #
@@ -293,10 +299,11 @@ def _echo(args):
     print(' '.join(args))
 aliases['echo'] = _echo
 
-echo --option1 \
+echo --option1 \\
 --option2
-""",
-        "--option1 --option2\n",
+echo missing \\
+EOL""",
+        "--option1 --option2\nmissing EOL\n",
         0,
     ),
     #
@@ -312,6 +319,34 @@ echo @$(which ls)
         0,
     ),
     #
+    # test @$() without leading/trailig WS
+    #
+    (
+        """
+def _echo(args):
+    print(' '.join(args))
+aliases['echo'] = _echo
+
+echo foo_@$(echo spam)_bar
+""",
+        "foo_spam_bar\n",
+        0,
+    ),
+    #
+    # test @$() outer product
+    #
+    (
+        """
+def _echo(args):
+    print(' '.join(args))
+aliases['echo'] = _echo
+
+echo foo_@$(echo spam sausage)_bar
+""",
+        "foo_spam_bar foo_sausage_bar\n",
+        0,
+    ),
+    #
     # test redirection
     #
     (
@@ -391,6 +426,49 @@ else:
         "Var foo\n",
         0,
     ),
+    #
+    # test logical subprocess operators
+    #
+    (
+        """
+def _echo(args):
+    print(' '.join(args))
+aliases['echo'] = _echo
+
+echo --version and echo a
+echo --version && echo a
+echo --version or echo a
+echo --version || echo a
+echo -+version and echo a
+echo -+version && echo a
+echo -+version or echo a
+echo -+version || echo a
+echo -~version and echo a
+echo -~version && echo a
+echo -~version or echo a
+echo -~version || echo a
+""",
+        """--version
+a
+--version
+a
+--version
+--version
+-+version
+a
+-+version
+a
+-+version
+-+version
+-~version
+a
+-~version
+a
+-~version
+-~version
+""",
+        0,
+    ),
 ]
 
 
@@ -529,6 +607,7 @@ def test_redirect_out_to_file(cmd, exp, tmpdir):
 @skip_if_no_xonsh
 @skip_if_no_sleep
 @skip_if_on_windows
+@pytest.mark.xfail(strict=False)  # TODO: fixme (super flaky on OSX)
 def test_xonsh_no_close_fds():
     # see issue https://github.com/xonsh/xonsh/issues/2984
     makefile = (
@@ -547,3 +626,67 @@ def test_xonsh_no_close_fds():
             f.write(makefile)
         out = sp.check_output(["make", "-sj2", "SHELL=xonsh"], universal_newlines=True)
         assert "warning" not in out
+
+
+@pytest.mark.parametrize(
+    "cmd, fmt, exp",
+    [("ls | wc", lambda x: x > "", True),],  # noqa E231 (black removes space)
+)
+def test_pipe_between_subprocs(cmd, fmt, exp):
+    "verify pipe between subprocesses doesn't throw an exception"
+    check_run_xonsh(cmd, fmt, exp)
+
+
+@skip_if_on_windows
+def test_negative_exit_codes_fail():
+    # see issue 3309
+    script = 'python -c "import os; os.abort()" && echo OK\n'
+    out, err, rtn = run_xonsh(script)
+    assert "OK" != out
+    assert "OK" != err
+
+
+@pytest.mark.parametrize(
+    "cmd, exp",
+    [
+        ("echo '&'", "&\n"),
+        ("echo foo'&'", "foo'&'\n"),
+        ("echo foo '&'", "foo &\n"),
+        ("echo foo '&' bar", "foo & bar\n"),
+    ],
+)
+def test_ampersand_argument(cmd, exp):
+    script = """
+#!/usr/bin/env xonsh
+def _echo(args):
+    print(' '.join(args))
+aliases['echo'] = _echo
+{}
+""".format(
+        cmd
+    )
+    out, _, _ = run_xonsh(script)
+    assert out == exp
+
+
+# issue 3402
+@skip_if_on_windows
+@pytest.mark.parametrize(
+    "cmd, exp_rtn",
+    [
+        ("sys.exit(0)", 0),
+        ("sys.exit(100)", 100),
+        ("sh -c 'exit 0'", 0),
+        ("sh -c 'exit 1'", 1),
+    ],
+)
+def test_single_command_return_code(cmd, exp_rtn):
+    _, _, rtn = run_xonsh(cmd, single_command=True)
+    assert rtn == exp_rtn
+
+
+@skip_if_on_msys
+@skip_if_on_windows
+@skip_if_on_darwin
+def test_argv0():
+    check_run_xonsh("checkargv0.xsh", None, "OK\n")
